#include <TimerOne.h>           // Avaiable from http://www.arduino.cc/playground/Code/Timer1
#include <SPI.h>
#include <MySensor.h>

#define NodeNum 11               // This is the hard coded Node Number and must be unique

#define SN "Device & Sw Module"
#define SV "1.0"

volatile boolean zero_cross = 0; // Flag to indicate we have crossed zero
int Dim_Step = 8;                    // step for count;
int freqStep = 75;             // This is the delay-per-brightness step in microseconds. It allows for 128 steps

//--------------------------------------------
struct device
{
  int Child_ID;
  byte AC_Pin;
  byte Sw_Current_State;							// Pin connected to triac
  byte Dim_Current_Level;
  byte Dim_Step_Counter;						//	Counts to dim State then switches Triac on NOT used for switches
  byte Requested_State;					//	requested dim State to go to as a %percentage
};
device Device[4];
//--------------------------------------------
MySensor gw;
 MyMessage DeviceMsg1(1, V_LIGHT);
MyMessage DeviceMsg2(2, V_LIGHT);
MyMessage DeviceMsg3(3, V_DIMMER);
MyMessage DeviceMsg4(4, V_DIMMER);
//--------------END MySensors Bits -----------
void setup()
{
  gw.begin(incomingMessage, NodeNum, true);
  gw.sendSketchInfo(SN, SV);                        // Send the Sketch Version Information to the Gateway

 IO_Setup();

 attachInterrupt(0, zero_cross_detect, RISING);    // Attach an Interrupt to Pin 2 (interrupt 0) for Zero Cross Detection
Timer1.initialize(freqStep);                      // Initialize TimerOne library for the freq we need
Timer1.attachInterrupt(dim_check, freqStep);      // Go to dim_check procedure every 75 uS (50Hz)  or 65 uS (60Hz) 128 times in 10mS for 128 step level control
}
//------------------------------------------------

void loop() {
  // Alway process incoming messages whenever possible
  gw.process();
}
 //------------------------------------------------------------------------
 // VOID INCOMING MESSAGE - VOID INCOMING MESSAGE - VOID INCOMING MESSAGE
 //------------------------------------------------------------------------

void incomingMessage(const MyMessage &message) {
byte x = 0;

  if (message.type == V_LIGHT) {
    byte i = message.sensor;         // holds the child ID
	Serial.print ("original x = ");
	Serial.print (x);
   x  = (message.getBool() ) >0 ? 1:0;
	Device[i].Requested_State = x;
	Serial.print(" Sensor =");
	Serial.print(i);
	Serial.print(" State =");
	Serial.print(x);
	Disp_Values("IO");
	
  }
  if (message.type == V_DIMMER) {
    int i = message.sensor;
    Device[i].Requested_State = atoi(message.data);     //data in payload should be ascii format eg 50 or 100
  }
}
 //------------------------------------------------------------------------
 //  END of INCOMING MESSAGE - 
 //------------------------------------------------------------------------

void zero_cross_detect() {
byte i;
  //Serial.println ("Got a zero cross");
  //Check if Switches should be on or off
  for ( i = 1; i < 3; i++) 
  {
    digitalWrite(Device[i].AC_Pin, Device[i].Requested_State);
    Device[i].Sw_Current_State =  Device[i].Requested_State;
  }
  for (i = 3; i < 5; i++)
  {
    digitalWrite(Device[i].AC_Pin, LOW);
    Device[i].Dim_Step_Counter = 0;
  }
  zero_cross = true;
  //Disp_Values ("Finished zero cross");
}
//***********************************************************************
// Turn on the TRIAC at the appropriate time
// We arrive here every 75 (65) uS
// Then check if the counter 'i' has reached the dimming State
// if so.... switch on the TRIAC each time this int occurs.
//This is OK as the continual On pulses won't upset anything
void dim_check() 
{
// Nothing here to cause problems
}
//-----------------------------------------------------------------------------------
void IO_Setup () {

  Device[1] = {1, A2, 0, 0, 0, 0};
  Device[2] = {2, A3, 0, 0, 0, 0};
  Device[3] = {3, A0, 1, 10, 0, 50}; 
  Device[4] = {4, A1, 0, 20, 0, 60};
  
  Serial.println(" Presenting childs");
  
    pinMode(Device[1].AC_Pin , OUTPUT); gw.present(Device[1].Child_ID, S_LIGHT);
    pinMode(Device[2].AC_Pin , OUTPUT); gw.present(Device[2].Child_ID, S_LIGHT);
 
    pinMode(Device[3].AC_Pin , OUTPUT); gw.present (Device[3].Child_ID, S_DIMMER);
	pinMode(Device[4].AC_Pin , OUTPUT); gw.present (Device[4].Child_ID, S_DIMMER);
 
  gw.send(DeviceMsg1.set(Device[1].Sw_Current_State));
  gw.send(DeviceMsg2.set(Device[2].Sw_Current_State));
  
  gw.send(DeviceMsg3.set (Device[3].Dim_Current_Level));
  gw.send(DeviceMsg4.set (Device[4].Dim_Current_Level));
  
  Disp_Values("IO");

}
//**************************************End IO Setup ********************************************************

void Disp_Values(char x[])
{
  Serial.print ("**** report on ");
  Serial.print (x);
  Serial.println (" ****");
  for (byte i = 1; i < 3; i++)
  {
    Serial.print("C:");
    Serial.print (Device[i].Child_ID);
    Serial.print(" P:");
    Serial.print (Device[i].AC_Pin);
    Serial.print(" SwCur:");
    Serial.print (Device[i].Sw_Current_State);
    Serial.print(" DReq:");
    Serial.println(Device[i].Requested_State);
  }

  for (byte i = 3; i < 5; i++)
  {
    Serial.print("C:");
    Serial.print (Device[i].Child_ID);
    Serial.print(" P:");
    Serial.print (Device[i].AC_Pin);
    Serial.print(" SwCur:");
    Serial.print (Device[i].Sw_Current_State);
    Serial.print(" DCur:");
    Serial.print (Device[i].Dim_Current_Level);
    Serial.print(" DCount:");
    Serial.print (Device[i].Dim_Step_Counter);
    Serial.print(" DReq:");
    Serial.println(Device[i].Requested_State);
  }
}
